<?php

namespace MF\BetterBlogsImporter\Admin\Controller;

use XenAddons\UBS\Entity\BlogEntryItem;
use XenAddons\UBS\Entity\BlogItem;
use XenAddons\UBS\Entity\Comment;
use XF\Admin\Controller\AbstractController;
use XF\Db\Schema\Create;
use XF\Mvc\ParameterBag;

class ImportBlogs extends AbstractController
{
    protected $findUsername = true;

    public function db()
    {
        return $this->app()->db();
    }

    public function phrase($phrase)
    {
        return \XF::phrase($phrase)->render();
    }

    public function actionIndex()
    {
        $db = $this->db();

        $sm = $db->getSchemaManager();

        $params = $this->checkRequirements();

        $params['entry_count'] = $db->fetchOne('SELECT COUNT(*) FROM `xfa_blog_entry`');

        $params['comment_count'] = $db->fetchOne('SELECT COUNT(*) FROM `xfa_blog_comment`');

        $confirm = $this->filter('confirm', 'bool');

        if ($this->isPost() && !$params['error'] && $confirm)
        {
            if (!$this->db()->getSchemaManager()->tableExists('mf_blog_import_log'))
            {
                $this->db()->getSchemaManager()->createTable('mf_blog_import_log', function (Create $table) {
                    $table->addColumn('import_id', 'int')->autoIncrement();
                    $table->addColumn('content_type', 'varchar')->length(25);
                    $table->addColumn('old_id', 'int')->setDefault(0);
                    $table->addColumn('new_id', 'int')->setDefault(0);
                    $table->addPrimaryKey('import_id');
                });
            }

            if (!$sm->tableExists('mf_import_resume'))
            {
                $sm->createTable('mf_import_resume', function (Create $table) {
                    $table->addColumn('resume_id', 'int')->autoIncrement();
                    $table->addColumn('data_type', 'varchar')->length(25);
                    $table->addColumn('id', 'int')->setDefault(0);
                    $table->addPrimaryKey('resume_id');
                });

                //If you want to import entry_read data then include ('entry_read', 0) to this query and uncomment entry_read related comments throughout the page.

                $db->query("INSERT INTO `mf_import_resume` (`data_type`, `id`) 
                                        VALUES 
                                    ('blog', 0),
                                    ('entry', 0),
                                    ('attachment_count', 0),
                                    ('reaction', 0),
                                    ('comment', 0),
                                    ('reaction_comment', 0),
                                    ('entry_watch', 0),
                                    ('blog_watch', 0),
                                    ('attachment', 0),
                                    ('import', 0)
                                    ");
            }

            return $this->actionImport(null, $this->getXfaCount(), $this->getUbsCount(), time());
        }

        return $this->view('', 'mf_betterblogsimporter', $params);
    }

    public function actionImport(ParameterBag $params = null, $xfaCount = null, $ubsCount = null, $launchTime = null)
    {
        $time = time();

        $limit = $this->filter('limit', 'uint');
        $maxLimit = $this->filter('maxLimit', 'uint');

        if (!$maxLimit)
        {
            $maxLimit = 100;
        }

        if (!$limit)
        {
            $limit = 1;
        }

        if (!$xfaCount)
        {
            $xfaCount = $this->filter('xfaCount', 'uint');

            if (!$xfaCount)
            {
                $xfaCount = $this->getXfaCount();
            }
        }

        if (!$ubsCount)
        {
            $ubsCount = $this->filter('ubsCount', 'uint');

            if (!$xfaCount)
            {
                $xfaCount = $this->getUbsCount();
            }

        }

        $resume = $this->db()->query('SELECT * FROM mf_import_resume')->fetchAll();

        if ($resume)
        {
            $viewParams['limit'] = $limit;

            foreach ($resume as $r)
            {
                $data_type = $r['data_type'];
                $id = $r['id'];

                switch ($data_type)
                {
                    case 'blog':
                        $progress = $this->processBlogs($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_creating_blogs');
                        break;
                    case 'entry':
                        $progress = $this->processEntries($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_importing_entries');
                        break;
                    case 'attachment_count':
                        $progress = $this->processAttachmentCount($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_processing_attachments');
                        break;
                    case 'reaction':
                        $progress = $this->processEntryReactions($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_processing_reactions');
                        break;
                    case 'comment':
                        $progress = $this->processComments($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_importing_comments');
                        break;
                    case 'reaction_comment':
                        $progress = $this->processCommentReactions($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_processing_comment_reactions');
                        break;
                    case 'entry_watch':
                        $progress = $this->processEntryWatch($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_importing_watched_entries');
                        break;
//                    case 'entry_read': $progress =  $this->processEntryRead($id, $limit);
//                        $viewParams['state'] = $this->phrase('mf_importing_read_entries');
//                        break;
                    case 'blog_watch':
                        $progress = $this->processBlogWatch($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_importing_watched_blogs');
                        break;
                    case 'attachment':
                        $progress = $this->processAttachments($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_processing_attachments');
                        break;
                    case 'import':
                        $progress = $this->processLogs($id, $limit);
                        $viewParams['state'] = $this->phrase('mf_finishing');
                        break;
                    default: $progress = null;
                }

                if ($this->calculateTime($time) > 5)
                {
                    switch ($limit)
                    {
                        case $limit > 10:
                            $viewParams['limit'] = $limit - 10;
                            break;
                        case $limit > 5:
                            $viewParams['limit'] = $limit - 5;
                            break;
                        default:
                            $viewParams['limit'] = $limit - 1;
                    }
                }

                if (!isset($progress['decreaseLimit']))
                {
                    if ($this->calculateTime($time) < 1)
                    {
                        $viewParams['limit'] = $limit + 1;
                    }
                }

                if ($viewParams['limit'] === 0)
                {
                    $viewParams['limit'] = 1;
                }

                if (is_array($progress))
                {
                    $this->updateRemaining($data_type, $progress['resume_id']);
                }

                if ($launchTime)
                {
                    $viewParams['launchTime'] = $launchTime;
                }
                else
                {
                    $viewParams['launchTime'] = $this->filter('launchTime', 'int');
                }

                $viewParams['time'] = time() - $viewParams['launchTime'];

                $minutes = floor($viewParams['time'] / 60);
                $secondsLeft = $viewParams['time'] % 60;

                if ($minutes < 10)
                {
                    $minutes = "0" . $minutes;
                }

                if ($secondsLeft < 10)
                {
                    $secondsLeft = "0" . $secondsLeft;
                }

                $viewParams['time'] = $secondsLeft;
                $viewParams['minutes'] = $minutes;

                if ($viewParams['limit'] > $maxLimit)
                {
                    $viewParams['limit'] = $maxLimit;
                }

                $viewParams['maxLimit'] = $maxLimit;

                if ($viewParams['launchTime'] == null)
                {
                    $viewParams['launchTime'] = time();
                }

                $viewParams['xfaCount'] = $xfaCount;
                $viewParams['ubsCount'] = $ubsCount + @$progress['ubsCount'];
                $viewParams['progress'] = $this->progressBar($xfaCount, $viewParams['ubsCount']);

                return $this->view('MF\BetterBlogsImporter:ImportBlogs', 'mf_import', $viewParams);
            }
        }

        return $this->message($this->phrase('mf_all_blogs_are_imported'));
    }

    public function processAttachmentCount($blog_entry_id, $limit)
    {
        $entries = $this->query("SELECT * FROM `xf_xa_ubs_blog_entry` WHERE blog_entry_id > ? ORDER BY `xf_xa_ubs_blog_entry`.`blog_entry_id` LIMIT ?", [$blog_entry_id, $limit])->fetchAll();

        if (!$entries)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'attachment_count'");
            return null;
        }

        foreach ($entries as $entry)
        {
            $content_id = $entry['blog_entry_id'];

            $attachment = $this->db()->query("SELECT * FROM `xf_attachment` WHERE content_type = 'ubs_blog_entry' AND content_id = ?", [$content_id])->fetchAll();

            $attach_count = count($attachment);

            $this->query("UPDATE `xf_xa_ubs_blog_entry` SET attach_count = ? WHERE blog_entry_id = ?", [$attach_count, $content_id]);
        }

        $end = end($entries);

        $progress['resume_id'] = $end['blog_entry_id'];
        $progress['ubsCount'] = count($entries);

        return $progress;

    }

    public function processLogs($blog_entry_id, $limit)
    {
        $entries = $this->query("SELECT * FROM `xf_xa_ubs_blog_entry` WHERE blog_entry_id > ? ORDER BY `xf_xa_ubs_blog_entry`.`blog_entry_id` LIMIT ?",
            [$blog_entry_id, $limit])->fetchAll();

        if (!$entries)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'import'");
            return null;
        }

        foreach ($entries as $entry)
        {
            $old_id = $entry['xfa_entry_id'];
            $new_id = $entry['blog_entry_id'];
            $content_type = 'entry';

            $this->query("INSERT IGNORE INTO `mf_blog_import_log` (`content_type`, `old_id`, `new_id`) VALUES (?, ?, ?)", [$content_type, $old_id, $new_id]);
        }

        $end = end($entries);

        $progress['resume_id'] = $end['blog_entry_id'];
        $progress['ubsCount'] = count($entries);

        return $progress;
    }

    public function processBlogs($user_id, $limit)
    {
        $blogs = $this->db()->query("SELECT * FROM xfa_blog WHERE entry_count > 0 AND user_id > ? ORDER BY `xfa_blog`.`user_id` LIMIT ?", [$user_id, $limit])->fetchAll();

        if (!$blogs)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'blog'");
            return null;
        }

        $index = 0;

        foreach ($blogs as $blog)
        {
            $blog['user_id'];
            $blog['entry_count'];
            $blog['view_count'];
            $blog['create_date'];
            $blog['last_entry'];
            $blog['last_entry_id'];
            $blog['blog_title'];
            $blog['description'];

            $query = $this->insertBlog($blog);
            $index++;

            if ($query === true)
            {
                $progress['resume_id'] = $blog['user_id'];
                $progress['ubsCount'] = $index;
                $progress['decreaseLimit'] = true;

                return $progress;
            }
        }

        $end = end($blogs);

        $progress['resume_id'] = $end['user_id'];
        $progress['ubsCount'] = count($blogs);

        return $progress;
    }

    public function processEntries($entry_id, $limit)
    {
        $entries = $this->db()->query("SELECT * FROM xfa_blog_entry WHERE entry_id > ? ORDER BY `xfa_blog_entry`.`entry_id` LIMIT ?", [$entry_id, $limit])->fetchAll();

        if (!$entries)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'entry'");
            return false;
        }

        $category_id = $this->finder('XenAddons\UBS:Category')->fetchOne()->getValue('category_id');

        foreach ($entries as $entry)
        {
            $entry['category_id'] = $category_id;

            $this->insertEntry($entry);
        }
        $end = end($entries);

        $progress['resume_id'] = $end['entry_id'];
        $progress['ubsCount'] = count($entries);

        return $progress;
    }

    public function processEntryReactions($reaction_id, $limit)
    {
        $reactions = $this->db()->fetchAll("SELECT * FROM `xf_reaction_content` WHERE content_type = 'xfa_blog_entry' AND reaction_content_id > ? ORDER BY `xf_reaction_content`.`reaction_content_id` LIMIT ?", [$reaction_id, $limit]);

        if (!$reactions)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'reaction'");
            return null;
        }

        foreach ($reactions as $reaction)
        {
            $content_id = $reaction['content_id'];
            $reaction_content_id = $reaction['reaction_content_id'];
            $new_content_id = $this->getEntryId($content_id);
            $content_type = 'ubs_blog_entry';

            $this->query("UPDATE IGNORE `xf_reaction_content` SET content_type = ?, content_id = ? WHERE content_type = 'xfa_blog_entry' AND reaction_content_id = ?", [$content_type, $new_content_id, $reaction_content_id]);
        }

        $end = end($reactions);

        $progress['resume_id'] = $end['reaction_content_id'];
        $progress['ubsCount'] = count($reactions);

        return $progress;
    }

    public function processCommentReactions($reaction_id, $limit)
    {
        $reactions = $this->db()->fetchAll("SELECT * FROM `xf_reaction_content` WHERE content_type = 'xfa_blog_comment' AND reaction_content_id > ? ORDER BY `xf_reaction_content`.`reaction_content_id` LIMIT ?", [$reaction_id, $limit]);

        if (!$reactions)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'reaction_comment'");
            return;
        }

        foreach ($reactions as $reaction)
        {
            $content_id = $reaction['content_id'];
            $reaction_content_id = $reaction['reaction_content_id'];
            $new_content_id = $this->getComment($content_id);
            $content_type = 'ubs_comment';

            $this->query("UPDATE IGNORE `xf_reaction_content` SET content_type = ?, content_id = ? WHERE content_type = 'xfa_blog_comment' AND reaction_content_id = ?", [$content_type, $new_content_id, $reaction_content_id]);
        }

        $end = end($reactions);

        $progress['resume_id'] = $end['reaction_content_id'];
        $progress['ubsCount'] = count($reactions);

        return $progress;
    }

    public function processComments($comment_id, $limit)
    {
        $comments = $this->db()->query("SELECT * FROM xfa_blog_comment WHERE comment_id > ? ORDER BY `xfa_blog_comment`.`comment_id` LIMIT ?", [$comment_id, $limit])->fetchAll();

        if (!$comments)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'comment'");
            return null;
        }

        $index = 0;

        foreach ($comments as $comment)
        {
            $query = $this->insertComment($comment);
            $index++;

            if ($query === true)
            {
                $progress['resume_id'] = $comment['comment_id'];
                $progress['ubsCount'] = $index;
                $progress['decreaseLimit'] = true;

                return $progress;
            }
        }

        $end = end($comments);

        $progress['resume_id'] = $end['comment_id'];
        $progress['ubsCount'] = count($comments);

        return $progress;
    }

    public function processEntryWatch($watch_id, $limit)
    {
        $entry_watch = $this->db()->query("SELECT * FROM xfa_blog_entry_watch WHERE watch_id > ? ORDER BY `xfa_blog_entry_watch`.`watch_id` LIMIT ?", [$watch_id, $limit])->fetchAll();

        if (!$entry_watch)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'entry_watch'");
            return null;
        }

        foreach ($entry_watch as $watch)
        {
            $user_id = $watch['user_id'];
            $blog_entry_id = $this->getEntryId($watch['entry_id']);
            $email_subscribe = 0;

            if ($blog_entry_id > 0)
            {
                $this->db()->query("INSERT IGNORE INTO `xf_xa_ubs_blog_entry_watch` (

                `user_id`,
                `blog_entry_id`,
                `email_subscribe`) 
                    VALUES (
                ?,
                ?,
                ?);
                ", [$user_id, $blog_entry_id, $email_subscribe]);
            }
        }
        $end = end($entry_watch);

        $progress['resume_id'] = $end['watch_id'];
        $progress['ubsCount'] = count($entry_watch);

        return $progress;
    }

    public function processEntryRead($read_id, $limit) //TODO:: FIX THIS FOR NONEXISTENT ENTRIES
    {
        $entry_read = $this->db()->query("SELECT * FROM xfa_blog_entry_read WHERE read_id > ? ORDER BY `xfa_blog_entry_read`.`read_id` LIMIT ?", [$read_id, $limit])->fetchAll();

        if (!$entry_read)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'entry_read'");

            return false;
        }

        foreach ($entry_read as $read)
        {
            $user_id = $read['user_id'];
            $blog_entry_id = $read['entry_id'] = $this->getEntryId($read['entry_id']);
            $blog_entry_read_date = $read['entry_read_date'];

            $this->db()->query("INSERT IGNORE INTO `xf_xa_ubs_blog_entry_read` (

                `user_id`,
                `blog_entry_id`,
                `blog_entry_read_date`) 
                    VALUES (
                ?,
                ?,
                ?);
                ", [$user_id, $blog_entry_id, $blog_entry_read_date]);
        }
        $end = end($entry_read);

        $progress['resume_id'] = $end['read_id'];
        $progress['ubsCount'] = count($entry_read);

        return $progress;
    }

    public function processBlogWatch($blog_watch_id, $limit)
    {
        $blog_watch = $this->db()->query("SELECT * FROM xfa_blog_watch WHERE watch_id > ? ORDER BY `xfa_blog_watch`.`watch_id` LIMIT ?", [$blog_watch_id, $limit])->fetchAll();

        if (!$blog_watch)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'blog_watch'");
            return false;
        }

        foreach ($blog_watch as $b_watch)
        {
            $user_id = $b_watch['user_id'];
            $blog_id = $this->getBlogId($b_watch['blog_user_id']);
            $notify_on = 'blog_entry';
            $send_alert = 1;
            $send_email = 0;

            if ($blog_id != 0)
            {
                $this->db()->query("INSERT IGNORE INTO `xf_xa_ubs_blog_watch` (
                    `user_id`,
                    `blog_id`,
                    `notify_on`,
                    `send_alert`,
                    `send_email`) 
                        VALUES (
                    ?,
                    ?,
                    ?,
                    ?,
                    ?);", [$user_id, $blog_id, $notify_on, $send_alert, $send_email]);

            }
        }

        $end = end($blog_watch);

        $progress['resume_id'] = $end['watch_id'];
        $progress['ubsCount'] = count($blog_watch);

        return $progress;
    }

    public function processAttachments($attach_id, $limit)
    {
        $attachment = $this->db()->query("SELECT * FROM `xf_attachment` WHERE content_type = 'xfa_blog_entry' AND attachment_id > ? ORDER BY `xf_attachment`.`attachment_id` LIMIT ?", [$attach_id, $limit])->fetchAll();

        if (!$attachment)
        {
            $this->query("DELETE FROM mf_import_resume WHERE data_type = 'attachment'");
            return;
        }

        $content_type = 'ubs_blog_entry';

        foreach ($attachment as $attach)
        {
            $content_id = $this->getEntryId($attach['content_id']);

            $attachment_id = $attach['attachment_id'];

            $this->db()->query("UPDATE `xf_attachment`
                    SET content_id = ?, content_type = ?
                    WHERE attachment_id = ?", [$content_id, $content_type, $attachment_id]);
        }

        $end = end($attachment);

        $progress['resume_id'] = $end['attachment_id'];
        $progress['ubsCount'] = count($attachment);

        return $progress;
    }

    protected function updateRemaining($data_type, $id)
    {
        if ($data_type && $id)
        {
            $this->query("UPDATE `mf_import_resume` SET id = ? WHERE data_type = ?", [$id, $data_type]);
        }
    }

    public function query($query, $params = [])
    {
        return $this->db()->query($query, $params);
    }

    protected function getXfaCount()
    {
        $xfaTables = $this->getXfaTables();
        $xfaCount = 0;

        foreach ($xfaTables as $table)
        {
            $xfaCount = $xfaCount + $this->db()->fetchOne("SELECT COUNT(*) FROM `$table`");
        }

        return $xfaCount;
    }

    protected function getUbsCount()
    {
        $ubsTables = $this->getUbsTables();
        $ubsCount = 0;

        foreach ($ubsTables as $table)
        {
            $ubsCount = $ubsCount + $this->db()->fetchOne("SELECT COUNT(*) FROM `$table`");
        }

        return $ubsCount;
    }

    public function progressBar($xfaCount = null, $ubsCount = null)
    {
        if (!$xfaCount || !$ubsCount)
        {
            return 0;
        }

        $progress = intval(($ubsCount / $xfaCount) * 100);

        if ($progress > 100)
        {
            return 100;
        }

        return $progress;
    }

    public function getXfaTables()
    {
        $tables[] = 'xfa_blog_watch';
//        $tables[] = 'xfa_blog_entry_read';
        $tables[] = 'xfa_blog_entry_watch';
        $tables[] = 'xfa_blog_comment';
        $tables[] = 'xfa_blog_entry';

        return $tables;
    }

    public function getUbsTables()
    {
        $tables[] = 'xf_xa_ubs_blog';
        $tables[] = 'xf_xa_ubs_blog_entry';
//        $tables[] = 'xf_xa_ubs_blog_entry_read';
        $tables[] = 'xf_xa_ubs_blog_entry_watch';
        $tables[] = 'xf_xa_ubs_blog_watch';
        $tables[] = 'xf_xa_ubs_comment';

        return $tables;
    }

    public function actionTruncate()
    {
        $visitor = \XF::visitor();

        if (!$visitor->is_super_admin)
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            $tables = $this->getUbsTables();

            foreach ($tables as $table)
            {
                $this->db()->query("TRUNCATE TABLE $table");
            }

            if ($this->db()->getSchemaManager()->tableExists('mf_blog_import_log'))
            {
                $this->db()->query('DROP TABLE mf_blog_import_log');
            }

            $this->query('DROP TABLE mf_import_resume');

            return $this->redirect('admin.php?betterblogsimporter', $this->phrase('mf_all_ubs_data_is_deleted'));
        }

        $viewParams = [

            'confirmUrl' => $this->buildLink('betterblogsimporter/truncate'),
            'contentTitle' => "Truncate all tables from User Blogs System? This will delete all the blogs, entries, comments etc. regardless of anything."
        ];

        return $this->view('XF:Delete/Delete', 'public:delete_confirm', $viewParams);
    }

    public function calculateTime($previousTime)
    {
        return time() - $previousTime;
    }

    public function getUsername($userId)
    {
        if (!$this->findUsername)
        {
            return [
                'username' => $this->phrase('guest'),
                'query' => false
            ];
        }

        $user = $this->app->find('XF:User', $userId);

        if ($user)
        {
            $result['username'] = $user->username;
            $result['query'] = false;
            return $result;
        }

        $db = $this->db();

        $blog = $db->query("SELECT `username` FROM `xf_xa_ubs_blog` WHERE user_id = ? LIMIT 1", [$userId])->fetch();

        if ($blog)
        {
            $result['username'] = $blog['username'];
            $result['query'] = false;
            return $result;
        }

        $comment = $db->query("SELECT `username` FROM `xf_xa_ubs_comment` WHERE user_id = ? LIMIT 1", [$userId])->fetch();

        if ($comment)
        {
            $result['username'] = $comment['username'];
            $result['query'] = false;
            return $result;
        }

        $post = $db->query("SELECT `message` FROM `xf_post` WHERE message LIKE '%USER=$userId%' OR message LIKE '%member: $userId%' LIMIT 1")->fetch();
        $result['query'] = true;

        if ($post)
        {
            if (preg_match('/(\[USER=.*]@(.*)\[\/.*]|\[.*="(.*),.*\[\/.*])/Uu', $post['message'], $match))
            {
                if ($match[2] == '')
                {
                    $result['username'] = $match[3];
                }
                else
                {
                    $result['username'] = $match[2];
                }
            }
        }

        if (!isset($result['username']))
        {
            $result['username'] = $this->phrase('guest');
        }

        return $result;
    }

    public function checkRequirements()
    {
        $addOns = \XF::app()->addOnManager()->getInstalledAddOns();

        if ($addOns['XenAddons/UBS'])
        {
            $xa_ubs = $addOns['XenAddons/UBS'];

            if (!$xa_ubs->getJsonVersion()['version_id'] == 2010670)
            {
                $warning = $this->phrase('mf_ubs_version_isnt_216');
            }

            $error = false;

        }
        else
        {
            $error = $this->phrase('mf_ubs_is_not_installed');
        }

        if (!$error && !isset($warning))
        {
            $success = $this->phrase('mf_you_may_continue');
        }
        else
        {
            $success = false;
        }

        if (isset($warning))
        {
            $viewParams['warning'] = $warning;
        }

        return $viewParams = [
            'success' => $success,
            'error' => $error
        ];
    }

    public function getBlogId($user_id)
    {
        $blog = $this->db()->query("SELECT blog_id FROM xf_xa_ubs_blog WHERE user_id = ?", [$user_id])->fetch();

        if (!$blog)
        {
            return 0;
        }

        return $blog['blog_id'];
    }

    public function getEntryId($entry_id)
    {
        $entry = $this->db()->query("SELECT blog_entry_id FROM xf_xa_ubs_blog_entry WHERE xfa_entry_id = ?", [$entry_id])->fetch();

        if (!$entry)
        {
            $entry['blog_entry_id'] = 0;
        }

        return $entry['blog_entry_id'];
    }

    public function getComment($comment_id)
    {
        $comment = $this->db()->query("SELECT comment_id FROM xf_xa_ubs_comment WHERE xfa_comment_id = ?", [$comment_id])->fetch();

        if (!$comment)
        {
            $comment['comment_id'] = 0;
        }

        return $comment['comment_id'];
    }

    public function insertComment($data)
    {
        /** @var Comment $comment */
        $comment = \XF::em()->create('XenAddons\UBS:Comment');

        $result = $this->getUsername($data['user_id']);

        $comment->bulkSet([
            'blog_entry_id' => $this->getEntryId($data['entry_id']),
            'user_id' => $data['user_id'],
            'username' => $result['username'],
            'comment_date' => $data['post_date'],
            'comment_state' => $data['message_state'],
            'message' => $data['message'],
            'reaction_score' => $data['reaction_score'],
            'ip_id' => $data['ip_id'],
            'last_edit_date' => $data['last_edit_date'],
            'last_edit_user_id' => $data['last_edit_user_id'],
            'edit_count' => $data['edit_count']
        ]);

        if ($comment->blog_entry_id)
        {
            $comment->save();
        }

        $this->db()->update('xf_xa_ubs_comment', [
            'xfa_comment_id' => $data['comment_id'],
            'reactions' => $data['reactions'],
            'reaction_users' => $data['reaction_users']
        ], "comment_id = ?", [$comment->comment_id]);

        if (isset($result['query']))
        {
            return false;
        }

        return true;
    }

    public function insertEntry($data)
    {
        /** @var BlogEntryItem $entry */
        $entry = \XF::em()->create('XenAddons\UBS:BlogEntryItem');

        $result = $this->getUsername($data['user_id']);

        $entry->user_id = $data['user_id'];
        $entry->category_id = $data['category_id'];
        $entry->blog_id = $this->getBlogId($data['user_id']);
        $entry->blog_entry_state = $data['message_state'];
        $entry->username = $result['username'];
        $entry->title = $data['title'];
        $entry->message = $data['message'];
        $entry->reaction_score = $data['reaction_score'];
        $entry->publish_date = $data['post_date'];
        $entry->view_count = $data['view_count'];
        $entry->last_update = $data['post_date'];
        $entry->edit_date = $data['post_date'];
        $entry->comments_open = $data['allow_comments'];
        $entry->comment_count = $data['reply_count'];
        $entry->last_edit_date = $data['last_edit_date'];
        $entry->edit_count = $data['edit_count'];
        $entry->ip_id = $data['ip_id'];
        $entry->last_edit_user_id = $data['last_edit_user_id'];

        $entry->save();

        $this->db()->update('xf_xa_ubs_blog_entry', [
            'xfa_entry_id' => $data['entry_id'],
            'reactions' => $data['reactions'],
            'reaction_users' => $data['reaction_users']
        ], "blog_entry_id = $entry->blog_entry_id");

        if (isset($result['query']))
        {
            return false;
        }

        return true;
    }

    public function insertBlog($data)
    {
        /** @var BlogItem $blog */
        $blog = \XF::em()->create('XenAddons\UBS:BlogItem');

        $result = $this->getUsername($data['user_id']);

        $blog->bulkSet([
            'user_id' => $data['user_id'],
            'username' => $result['username'],
            'title' => $data['blog_title'] ? $data['blog_title'] : 'Blog: ' . $result['username'],
            'message' => $data['blog_title'] ? $data['blog_title'] : 'Blog: ' . $result['username'],
            'create_date' => $data['create_date'],
            'view_count' => $data['view_count'],
            'last_update' => $data['last_entry'],
            'blog_entry_count' => $data['entry_count'],
            'last_blog_entry_date' => $data['last_entry'],
            'last_blog_entry_id' => $data['last_entry_id']
        ]);

        $blog->save();

        if (isset($result['query']))
        {
            return false;
        }

        return true;
    }

}